import ckan.plugins as plugins
import ckan.plugins.toolkit as toolkit
import ckan.authz as authz

from ckan.lib.mailer import mail_recipient, MailerException
from ckan.logic.action.create import user_create

from ckan.logic import side_effect_free, check_access
from ckan.logic.action.get import package_show, resource_show

from ckanext.restricted import helpers
from ckanext.restricted import logic

from pylons import config
import simplejson as json

from logging import getLogger
log = getLogger(__name__)

def restricted_user_create_and_notify(context, data_dict):

    def body_from_user_dict(user_dict):
         body = u'\n'
         for key,value in user_dict.items():
             body +=  ' \t - '+ key.upper() + ': ' + ( value if type(value) == str else unicode(value)) + '\n'
         return body
    user_dict = user_create(context, data_dict)

    # Send your email, check ckan.lib.mailer for params
    try:
        name = 'CKAN System Administrator'
        email = config.get('email_to')
        subject = u'New Registration: ' +  user_dict.get('name', 'new user') + ' (' +  user_dict.get('email') + ')'
        body = u'A new user registered, please review the information: ' + body_from_user_dict(user_dict)
        log.debug('Mail sent to ' + email + ', subject: ' + subject)
        mail_recipient(name, email, subject, body)

    except MailerException as mailer_exception:
        log.error("Cannot send mail after registration ")
        log.error(mailer_exception)
        pass

    return (user_dict)

@side_effect_free
def restricted_package_show(context, data_dict):
    package_metadata = package_show(context, data_dict)

    # Ensure user who can edit can see the resource
    if authz.is_authorized('package_update', context, package_metadata).get('success', False):
        return package_metadata

    # Custom authorization
    if (type(package_metadata) == type(dict())):
        restricted_package_metadata = dict(package_metadata)
    else:
        package_metadata_json_str = str(package_metadata.for_json())
        restricted_package_metadata = json.loads(package_metadata_json_str)

    restricted_resources_list = []
    for resource in restricted_package_metadata.get('resources',[]):
        authorized = restricted_resource_show(context, {'id':resource.get('id'), 'resource':resource, 'package': package_metadata }).get('success', False)
        log.debug('restricted_package_show ' + resource.get('id','') + ', ' + resource.get('name','') + ' (' + str(resource.get('restricted', '')) + '): ' + str(authorized))
        restricted_resource = dict(resource)
        log.debug(restricted_resource)
        if not authorized:
            restricted_resource['url'] = 'Not Authorized'
        restricted_resources_list += [restricted_resource]
    restricted_package_metadata['resources'] = restricted_resources_list

    return (restricted_package_metadata)

@toolkit.auth_allow_anonymous_access
def restricted_resource_show(context, data_dict=None):

    # Ensure user who can edit can see the resource
    resource = data_dict.get('resource', context.get('resource',{}))
    if type(resource) is not dict:
        resource = resource.as_dict()

    if authz.is_authorized('package_update', context, {'id': resource.get('package_id')}).get('success'):
        return ({'success': True })

    # custom retricted check
    auth_user_obj = context.get('auth_user_obj', None)
    user_name = ""
    if auth_user_obj:
        user_name = auth_user_obj.as_dict().get('name','')
    else:
        if authz.get_user_id_for_username(context.get('user'), allow_none=True):
            user_name = context.get('user','')
    log.debug("restricted_resource_show: USER:" + user_name)

    package = data_dict.get('package', {})
    if not package:
        model = context['model']
        package = model.Package.get(resource.get('package_id'))
        package = package.as_dict()

    return (logic.restricted_check_user_resource_access(user_name, resource, package))

class RestrictedPlugin(plugins.SingletonPlugin):
    plugins.implements(plugins.IConfigurer)
    plugins.implements(plugins.IActions)
    plugins.implements(plugins.ITemplateHelpers)
    plugins.implements(plugins.IAuthFunctions)
    plugins.implements(plugins.IRoutes, inherit=True)

    # IConfigurer

    def update_config(self, config_):
        toolkit.add_template_directory(config_, 'templates')
        toolkit.add_public_directory(config_, 'public')
        toolkit.add_resource('fanstatic', 'restricted')

    # IActions

    def get_actions(self):
        return { 'user_create': restricted_user_create_and_notify,
                 'package_show': restricted_package_show
                 #,'restricted_request_access': restricted_request_access
        }

    # ITemplateHelpers

    def get_helpers(self):
        return { 'restricted_get_user_id':helpers.restricted_get_user_id}

    # IAuthFunctions

    def get_auth_functions(self):
        return { 'resource_show': restricted_resource_show,
                 #'resource_view_list': restricted_resource_show
                 'resource_view_show': restricted_resource_show
               }
    # IRoutes

    def before_map(self, map_):
        map_.connect(
            'restricted_request_access',
            '/dataset/{package_id}/restricted_request_access/{resource_id}',
            controller='ckanext.restricted.controller:RestrictedController',
            action = 'restricted_request_access_form'
        )
        return map_



import logging
import sys

import ckan.logic as logic

import json


logger = logging.getLogger(__name__)


def restricted_check_user_resource_access(user, resource_dict, package_dict):
    restricted_level = 'public'
    allowed_users  = []

    # check in resource_dict
    if resource_dict:
        extras = resource_dict.get('extras',{})
        restricted = resource_dict.get('restricted', extras.get('restricted', {}))
        if not isinstance(restricted, dict):
            try:
                restricted = json.loads(restricted)
            except:
                print("Unexpected error:", sys.exc_info()[0])
                restricted = {}
        restricted_level = restricted.get('level', 'public')
        allowed_users = restricted.get('allowed_users', "").split(',')

    # Public resources (DEFAULT)
    if not restricted_level or restricted_level == 'public':
        return {'success': True }

    # Registered user
    if not user:
        return {'success': False, 'msg': "Resource access restricted to registered users" }
    else:
        if restricted_level == 'registered' or not restricted_level:
            return {'success': True }

    # Since we have a user, check if it is in the allowed list
    if user in allowed_users:
        return {'success': True }

    # Get organization list
    user_organization_dict = {}

    context = {'user': user}
    data_dict = {'permission': 'read'}

    for org in logic.get_action('organization_list_for_user')(context, data_dict):
        name = org.get('name', '')
        id = org.get('id', '')
        if name and id:
            user_organization_dict[id] =  name

    # Any Organization Members (Trusted Users)
    if not user_organization_dict:
        return {'success': False, 'msg': "Resource access restricted to members of an organization" }
    if restricted_level == 'any_organization':
        return {'success': True }

    pkg_organization_id = package_dict.get('owner_org', '')

    # Same Organization Members
    if restricted_level == 'same_organization':
        if pkg_organization_id in user_organization_dict.keys():
            return {'success': True }

    return {'success': False, 'msg': "Resource access restricted to same organization (" + pkg_organization_id + ") members" }




import ckan.model as model
import ckan.logic as logic
import ckan.lib.base as base
import ckan.lib.helpers as h
import ckan.lib.captcha as captcha
import ckan.lib.mailer as mailer

import ckan.plugins.toolkit as toolkit
from ckan.common import _, request, c, g
from routes import url_for

from logging import getLogger
from pylons import config
from email.header import Header

import simplejson as json

import ckan.lib.navl.dictization_functions as dictization_functions
DataError = dictization_functions.DataError
unflatten = dictization_functions.unflatten

render = base.render

log = getLogger(__name__)

class RestrictedController(toolkit.BaseController):

    def __before__(self, action, **env):
        base.BaseController.__before__(self, action, **env)
        try:
            context = {'model': base.model, 'user': base.c.user or base.c.author,
                       'auth_user_obj': base.c.userobj}
            logic.check_access('site_read', context)
        except logic.NotAuthorized:
            base.abort(401, _('Not authorized to see this page'))

    def _send_request_mail(self, data):
        try:
            site_title = g.site_title
            email_dict = {data.get('maintainer_email'): data.get('maintainer_name', 'Maintainer'), config.get('email_to'): site_title + ' Admin'}
            subject = 'Access Request to resource ' +  data.get('resource_name','') + ' (' +  data.get('package_name','')  + ') from ' + data.get('user_name','')
            url = config.get('ckan.site_url') + url_for(controller='package', action='resource_read', id=data.get('package_name') , resource_id=data.get('resource_id'))
            edit_link = config.get('ckan.site_url') + url_for(controller='package', action='resource_edit', id=data.get('package_name') , resource_id=data.get('resource_id'))
            body = 'A user has requested access to your data in ' + site_title + ': '
            body += '\n\t * Resource: ' +  data.get('resource_name','') + ' ( ' + str(url) + ' )'
            body += '\n\t * Dataset: ' +  data.get('package_name','')
            body += '\n\t * User: ' + data.get('user_name','') + ' (' + data.get('user_email','') + ')'
            body += '\n\t * Message: ' + data.get('message','')
            body += '\n\n You can allow this user to access you resource by adding ' + data.get('user_id','the user id')+ ' to the list of allowed users.'
            body += ' If you have editor rights, you can edit the resource in this link: ' + str(edit_link)
            body += '\n\n If you have any questions about how to preceed with this request, please contact the ' + site_title + ' support at ' + config.get('email_to')

            headers = {'CC': ",".join(email_dict.keys()),  'reply-to': data.get('user_email')}
            ## CC doesn't work and mailer cannot send to multiple addresses
            for email, name in email_dict.iteritems():
                mailer.mail_recipient(name, email, subject, body, headers)
            ## Special copy for the user (no links)
            email = data.get('user_email')
            name = data.get('user_name','User')
            body_user = "Please find below a copy of the access request mail sent. \n\n >> " + body.replace("\n", "\n >> ").replace(edit_link, " [ ... ] ").replace(url, " [ ... ] ")
            mailer.mail_recipient(name, email, subject, body_user, headers)

        except mailer.MailerException as mailer_exception:
            log.error("Cannot access request mail after registration ")
            log.error(mailer_exception)
            pass

        return

    def _send_request(self, context):

        try:
            data_dict = logic.clean_dict(unflatten(
                logic.tuplize_dict(logic.parse_params(request.params))))

            captcha.check_recaptcha(request)

        except logic.NotAuthorized:
            toolkit.abort(401, _('Not authorized to see this page'))
        except captcha.CaptchaError:
            error_msg = _(u'Bad Captcha. Please try again.')
            h.flash_error(error_msg)
            return self.restricted_request_access_form(package_id=data_dict.get('package_name'), resource_id=data_dict.get('resource'), data=data_dict)

        try:
            pkg = toolkit.get_action('package_show')(context, {'id': data_dict.get('package_name')})
            data_dict['pkg_dict'] = pkg
        except toolkit.ObjectNotFound:
            toolkit.abort(404, 'Dataset not found')
        except:
            toolkit.abort(404, 'Exception retrieving dataset to send mail')

        # Validation
        errors = {}
        error_summary = {}

        if (data_dict["message"] == ''):
            errors['message'] = [u'Missing Value']
            error_summary['message'] =  u'Missing Value'

        if len(errors) > 0:
            return self.restricted_request_access_form(package_id=data_dict.get('package-name'), resource_id=data_dict.get('resource'), errors=errors, error_summary=error_summary, data=data_dict)

        log.debug("TODO: Send Mail")
        self._send_request_mail(data_dict)

        return render('restricted/restricted_request_access_result.html', extra_vars={'data': data_dict, 'pkg_dict': pkg } )

    def restricted_request_access_form(self, package_id, resource_id, data=None, errors=None, error_summary=None):
        '''Redirects to form
        '''
        user_id = toolkit.c.user
        context = {
            'model': model,
            'session': model.Session,
            'user': user_id,
            'save': 'save' in request.params
        }

        data = data or {}
        errors = errors or {}
        error_summary = error_summary or {}

        if (context['save']) and not data and not errors:
            return self._send_request(context)

        if not data:
            data['package_id'] = package_id
            data['resource_id'] = resource_id

            user = toolkit.get_action('user_show')(context, {'id': user_id})
            data['user_id'] = user_id
            data['user_name'] = user.get('display_name', user_id)
            data['user_email'] = user.get('email', '')

            resource_name = ""

            try:
                pkg = toolkit.get_action('package_show')(context, {'id': package_id})
                data['package_name'] = pkg.get('name')
                resources = pkg.get('resources', [])
                for resource in resources:
                    if resource['id'] == resource_id:
                        resource_name = resource['name']
                        break
                else:
                    toolkit.abort(404, 'Dataset resource not found')
                # get mail
                contact_email = pkg.get('maintainer_email', "")
                contact_name = pkg.get('maintainer_name', "Dataset Maintainer")
                if not contact_email:
                    contact_email = json.loads(pkg.get('maintainer', "{}")).get('email','')
                    contact_name = json.loads(pkg.get('maintainer', "{}")).get('name','Dataset Maintainer')
                if not contact_email:
                    contact_email = config.get('email_to')
                    contact_name = "CKAN Admin"
            except toolkit.ObjectNotFound:
                toolkit.abort(404, 'Dataset not found')
            except:
                toolkit.abort(404, 'Exception retrieving dataset for the form')

            data['resource_name'] = resource_name
            data['maintainer_email'] = contact_email
            data['maintainer_name'] = contact_name
        else:
            pkg = data.get('pkg_dict', {})

        extra_vars = {'pkg_dict':pkg, 'data': data, 'errors':errors, 'error_summary': error_summary}
        return render('restricted/restricted_request_access_form.html', extra_vars=extra_vars)
